////////////////////////////////////////////////////////////////////////
// Thieves Game
// The beginners
////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "sfx/sound.h"
#include "cpcrslib.h"
#include "cpcwyzlib.h"
#include "sprites.h"


#define TILE_HEIGHT 10	//pixel
#define TILE_WIDTH 8	//pixel (4 bytes)	
#define MODE0_HEIGHT 200
#define MODE0_WIDTH 160
#define NUM_TILES_WIDTH MODE0_WIDTH / TILE_WIDTH
#define NUM_TILES_HEIGHT MODE0_HEIGHT / TILE_HEIGHT
#define MAX_NUM_COIN 4
#define MAX_NUM_FRAMES 20
#define NUM_COLORS 11

#define SCORE_Y 20
#define SCORE_X_1 MODE0_WIDTH
#define SCORE_X_2 MODE0_WIDTH * 3
#define MID_SCREEN MODE0_WIDTH * 2

// Teclas
#define KEY_IZQ  0
#define KEY_DER  1
#define KEY_ARR  2
#define KEY_ABA  3
#define KEY_A 	 4
#define KEY_W 	 5
#define KEY_D 	 6
#define KEY_S 	 7
#define KEY_1 	 8
#define KEY_2 	 9
#define KEY_3    10
#define KEY_4 	 11
#define KEY_FIR  12
#define KEY_RET	 13
#define KEY_ENT  14
#define KEY_ESC  15

const unsigned char Palette[NUM_COLORS] = {0, 26, 25, 18, 15, 9, 6, 22, 3, 2, 11};


//Tipos de tiles que van a haber en la escena
enum _eTileType
{
	TileType_None,
	TileType_Border,
	TileType_Rock,
	TileType_Coin,
	TileType_Coin_3,
	TileType_Coin_5,
	TileType_Periz1,
	TileType_Perde1,
	TileType_Periz2,
	TileType_Perde2
}_eTileType;

//Matriz escenario
enum _eTileType aBackgroundTiles[NUM_TILES_WIDTH][NUM_TILES_HEIGHT]; //20x20

typedef struct _tSprite
{
  unsigned char nY;
  unsigned char nX;
  unsigned int nCoins;
  unsigned char p;
}_tSprite;
typedef struct _tSprite **tSpriteArrayPtr;
typedef struct _tSprite *tSpritePtr;
typedef struct _tFrame
{
  unsigned char nY;
  unsigned char nX;
}_tFrame;


//ANIMACIONES
#define K_NUM_ANIMATIONS 5
char *aKeyFrames[K_TYPE_COINS][K_NUM_ANIMATIONS];
unsigned char numFrames = 1;
unsigned char tableCoins[NUM_TILES_WIDTH][NUM_TILES_HEIGHT];

_tFrame aFrames[MAX_NUM_FRAMES];

//Monedas
_tSprite aCoins[MAX_NUM_COIN];
tSpriteArrayPtr aCoinsPtr;

//Personaje
_tSprite per1,per2;
//Modo
int modo=1;
int dif=1;
////////////////////////////////////////////////////////////////////////
//GetTime()
////////////////////////////////////////////////////////////////////////
unsigned char char3,char4;
unsigned int GetTime()
{
	unsigned int nTime = 0;

	__asm
		CALL #0xBD0D ;KL TIME PLEASE
		PUSH HL
		POP DE
		LD HL, #_char3
		LD (HL), D
		LD HL, #_char4
		LD (HL), E
	__endasm;

	nTime = (char3 << 8) + char4;

	return nTime;
}
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
//PutSpriteMode0()
////////////////////////////////////////////////////////////////////////
void PutSpriteMode0(unsigned char *pAddress, unsigned char nWidth, unsigned char nHeight, unsigned char *pSprite)
{
	__asm
		LD L, 4(IX) 
		LD H, 5(IX) 
		LD C, 6(IX) 
		LD B, 7(IX)            
		LD E, 8(IX) 
		LD D, 9(IX) 

		_loop_alto:
			PUSH BC
			LD B,C
			PUSH HL
		_loop_ancho:
			LD A,(DE)
			LD (HL),A
			INC DE
			INC HL
			DJNZ _loop_ancho
			POP HL
			LD A,H
			ADD #0x08
			LD H,A
			SUB #0xC0
			JP NC, _sig_linea
			LD BC, #0xC050
			ADD HL,BC
		_sig_linea:
			POP BC
			DJNZ _loop_alto
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//DrawTile
////////////////////////////////////////////////////////////////////////
void DrawTile(unsigned char nTileX, unsigned char nTileY)
{
	enum _eTileType eTileType = aBackgroundTiles[nTileX][nTileY];

	unsigned int nY = nTileY * TILE_HEIGHT;
	unsigned int nX = nTileX * TILE_WIDTH;
	unsigned char nColor=0;
	unsigned char *pScreen = (unsigned char *)0xC000 + ((nY / 8) * 80) + ((nY % 8) * 2048) + (nX / 2);
	unsigned char *pSprite = (unsigned char *)SpriteSand2;

	switch(eTileType)
	{
		case TileType_Border:
			pSprite = (unsigned char *)SpriteBrick;
			break;
		case TileType_Rock:
			pSprite = (unsigned char *)SpriteRock;
			break;
		case TileType_Coin:		    
			pSprite = (unsigned char *)aKeyFrames[0][numFrames];			
			break;	
		case TileType_Coin_3:		    
			pSprite = (unsigned char *)aKeyFrames[1][numFrames];			
			break;
		case TileType_Coin_5:		    
			pSprite = (unsigned char *)aKeyFrames[2][numFrames];			
			break;					
		case TileType_Periz1:		    
			pSprite = (unsigned char *)LadronQuietoIzqSprite1;			
			break;	
		case TileType_Perde1:		    
			pSprite = (unsigned char *)LadronQuietoDerSprite1;			
			break;	
		case TileType_Periz2:		    
			pSprite = (unsigned char *)LadronQuietoIzqSprite2;			
			break;	
		case TileType_Perde2:		    
			pSprite = (unsigned char *)LadronQuietoDerSprite2;			
			break;	
	}		
	
	PutSpriteMode0(pScreen, TILE_WIDTH/2, TILE_HEIGHT, pSprite);
}
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
//SetColor
////////////////////////////////////////////////////////////////////////
void SetColor(unsigned char nColorIndex, unsigned char nPaletteIndex)
{
	__asm
		ld a, 4 (ix)
		ld b, 5 (ix)
		ld c, b
		call #0xBC32 ;SCR SET INK
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetMode
////////////////////////////////////////////////////////////////////////
void SetMode(unsigned char nMode)
{
	__asm
		ld a, 4 (ix)
		call #0xBC0E ;SCR_SET_MODE
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetCursor
////////////////////////////////////////////////////////////////////////
void SetCursor(unsigned char nColum, unsigned char nLine)
{
	__asm
		ld h, 4 (ix)
		ld l, 5 (ix)
		call #0xBB75 ;TXT SET CURSOR
	__endasm;
}
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
//InitGame
////////////////////////////////////////////////////////////////////////
void InitGame()
{	
	unsigned int nX = 0;
	unsigned int nY = 0;
	unsigned char nColor = 0;
	
	cpc_SetMode(0);   //mode change. The interrupt standard jump must be disabled.
    cpc_SetModo(0);
	
	//init personajes
	per1.nX=5;
	per1.nY=NUM_TILES_HEIGHT - 2;
	per1.nCoins=0;
	per1.p=1;
	per2.nX=10;
	per2.nY=NUM_TILES_HEIGHT - 2;
	per2.nCoins=0;	
	per2.p=2;
	
	//Establece los colores de la tableta de colores
	for(nColor = 0; nColor < NUM_COLORS; nColor++)
		SetColor(nColor, Palette[nColor]);

	for(nX = 0; nX < NUM_TILES_WIDTH; nX++)
		for(nY = 0; nY < NUM_TILES_HEIGHT; nY++){
			aBackgroundTiles[nX][nY] = TileType_None;
			tableCoins[nX][nY]=0;
		}

	//Establecemos que los tiles de los bordes de la pantallas seran de tipo borde
	for(nX = 0; nX < NUM_TILES_WIDTH; nX++){
		aBackgroundTiles[nX][0] = TileType_Border;
		aBackgroundTiles[nX][NUM_TILES_HEIGHT - 1] = TileType_Border;
	}

	for(nY = 0; nY < NUM_TILES_HEIGHT; nY++){
		aBackgroundTiles[0][nY] = TileType_Border;
		aBackgroundTiles[NUM_TILES_WIDTH - 1][nY] = TileType_Border;
	}
	//Pinto el personaje 1	
	aBackgroundTiles[per1.nX][per1.nY] = TileType_Perde1;
	aBackgroundTiles[per2.nX][per2.nY] = TileType_Perde2;
		
	aKeyFrames[0][0]=(char *)SpriteCoin[0];
	aKeyFrames[0][1]=(char *)SpriteCoin[0] + (TILE_WIDTH/2) * TILE_HEIGHT;
	aKeyFrames[0][2]=(char *)SpriteCoin[0] + (TILE_WIDTH/2 * TILE_HEIGHT)*2;
	aKeyFrames[0][3]=(char *)SpriteCoin[0] + (TILE_WIDTH/2 * TILE_HEIGHT)*3;
	aKeyFrames[0][4]=(char *)SpriteCoin[0] + (TILE_WIDTH/2 * TILE_HEIGHT)*4;	
	aKeyFrames[1][0]=(char *)SpriteCoin[1];
	aKeyFrames[1][1]=(char *)SpriteCoin[1] + (TILE_WIDTH/2) * TILE_HEIGHT;
	aKeyFrames[1][2]=(char *)SpriteCoin[1] + (TILE_WIDTH/2 * TILE_HEIGHT)*2;
	aKeyFrames[1][3]=(char *)SpriteCoin[1] + (TILE_WIDTH/2 * TILE_HEIGHT)*3;
	aKeyFrames[1][4]=(char *)SpriteCoin[1] + (TILE_WIDTH/2 * TILE_HEIGHT)*4;	
	aKeyFrames[2][0]=(char *)SpriteCoin[2];
	aKeyFrames[2][1]=(char *)SpriteCoin[2] + (TILE_WIDTH/2) * TILE_HEIGHT;
	aKeyFrames[2][2]=(char *)SpriteCoin[2] + (TILE_WIDTH/2 * TILE_HEIGHT)*2;
	aKeyFrames[2][3]=(char *)SpriteCoin[2] + (TILE_WIDTH/2 * TILE_HEIGHT)*3;
	aKeyFrames[2][4]=(char *)SpriteCoin[2] + (TILE_WIDTH/2 * TILE_HEIGHT)*4;	
		
		
	//Creamos el vector con las monedas y las inicializamos
	aCoinsPtr=(struct _tSprite **)malloc(sizeof(struct _tSprite)*MAX_NUM_COIN);	
	for(nX = 0; nX < MAX_NUM_COIN; nX++)
		aCoinsPtr[nX]=NULL;			

	//Pintamos el escenario en pantalla
	for(nX = 0; nX < NUM_TILES_WIDTH; nX++)
		for(nY = 0; nY < NUM_TILES_HEIGHT; nY++)
			DrawTile(nX, nY);					
}
////////////////////////////////////////////////////////////////////////

void ShowMenuWinner(unsigned int pPlayer1, unsigned int pPlayer2)
{
  SetMode(1);

  SetCursor(10, 5);
  printf("Thief vs Thief");

  SetCursor(14, 8);
  if(pPlayer1>pPlayer2)
  {
  	printf("Player 1 won!\n\rNo one steal more than you! Bribon!");
  }
  else if(pPlayer2>pPlayer1)
  {
  	printf("Player 2 won!\n\rNo one steal more than you! Paela!");
  }
  else
  {
	printf("It's a DRAW!\n\rAre you hacking, ma'am?");
  }

  SetCursor(4, 16);
  printf("Press 'ENTER' to play again");
  cpc_ScanKeyboard();
  while(!cpc_TestKeyF(KEY_ENT)){ cpc_ScanKeyboard(); }
}
void throwCharLeftP1(signed int beingHitted)
{
	if(beingHitted==100)
	{
		if(per1.nX>1)
		{
			aBackgroundTiles[per1.nX][per1.nY] = TileType_None;
			DrawTile(per1.nX,per1.nY);
			per1.nX=per1.nX-1;
			aBackgroundTiles[per1.nX][per1.nY] = TileType_Perde1;
			DrawTile(per1.nX,per1.nY);
		}
	}
}

void throwCharRightP1(signed int beingHitted)
{
	if(beingHitted==-100)
	{
		if(per1.nX+1<NUM_TILES_WIDTH-1)
		{
			aBackgroundTiles[per1.nX][per1.nY] = TileType_None;
			DrawTile(per1.nX,per1.nY);
			per1.nX=per1.nX+1;
			aBackgroundTiles[per1.nX][per1.nY] = TileType_Periz1;
			DrawTile(per1.nX,per1.nY);
		}
	}
}

void throwCharLeftP2(signed int beingHitted)
{
	if(beingHitted==100)
	{
		if(per2.nX>1)
		{
			aBackgroundTiles[per2.nX][per2.nY] = TileType_None;
			DrawTile(per2.nX,per2.nY);
			per2.nX=per2.nX-1;
			aBackgroundTiles[per2.nX][per2.nY] = TileType_Perde2;
			DrawTile(per2.nX,per2.nY);
		}
	}
}

void throwCharRightP2(signed int beingHitted)
{
	if(beingHitted==-100)
	{
		if(per2.nX+1<NUM_TILES_WIDTH-1)
		{
			aBackgroundTiles[per2.nX][per2.nY] = TileType_None;
			DrawTile(per2.nX,per2.nY);
			per2.nX=per2.nX+1;
			aBackgroundTiles[per2.nX][per2.nY] = TileType_Periz2;
			DrawTile(per2.nX,per2.nY);
		}
	}
}

int actionPunchP2()
{
	enum _eTileType eTileType = aBackgroundTiles[per2.nX][per2.nY];
	if(TileType_Periz2 && per1.nX-1==per2.nX)
	{
		//hacer que el personaje haga como que pega un puñetazo hacia la izquierda
		return -100;
	}
	else if(TileType_Perde2 && per1.nX+1==per2.nX)
	{
		//hacer que el personaje haga como que pega un puñetazo hacia la derecha
		return +100;
	}
	return 0;
}

int actionPunchP1()
{
	enum _eTileType eTileType = aBackgroundTiles[per1.nX][per1.nY];
	if(TileType_Periz1 && per2.nX-1==per1.nX)
	{
		//hacer que el personaje haga como que pega un puñetazo hacia la izquierda
		return -100;
	}
	else if(TileType_Perde1 && per2.nX+1==per1.nX)
	{
		//hacer que el personaje haga como que pega un puñetazo hacia la derecha
		return +100;
	}
	return 0;
}

void jump(_tSprite * per){
	if ((cpc_WyzTestPlayer()&8)==0){
		cpc_WyzStartEffect(1,4);
	}
	aBackgroundTiles[per->nX][per->nY] = TileType_None;				
	DrawTile(per->nX,per->nY);
	per->nY=per->nY-1;
	aBackgroundTiles[per->nX][per->nY] = (per->p==1)?TileType_Periz1 : TileType_Periz2;;
	DrawTile(per->nX,per->nY);
}

void normal(_tSprite * per){
	enum _eTileType eTileTypeAux=aBackgroundTiles[per->nX][per->nY];
	aBackgroundTiles[per->nX][per->nY] = TileType_None;				
	DrawTile(per->nX,per->nY);
	per->nY=per->nY+1;
	aBackgroundTiles[per->nX][per->nY] = eTileTypeAux;
	DrawTile(per->nX,per->nY);
}

void move_left(_tSprite * per, unsigned char nTiles){
	aBackgroundTiles[per->nX][per->nY] = TileType_None;
	DrawTile(per->nX,per->nY);
	per->nX=per->nX-nTiles;
	aBackgroundTiles[per->nX][per->nY] = (per->p==1)?TileType_Periz1 : TileType_Periz2;
	DrawTile(per->nX,per->nY);
}

void move_right(_tSprite * per, unsigned char nTiles){
	aBackgroundTiles[per->nX][per->nY] = TileType_None;
	DrawTile(per->nX,per->nY);
	per->nX=per->nX+nTiles;
	aBackgroundTiles[per->nX][per->nY] = (per->p==1)?TileType_Perde1 : TileType_Perde2;
	DrawTile(per->nX,per->nY);
}

/*
unsigned char isEmptyCol(unsigned char col){
	unsigned char i;
	for(i = 0; i < MAX_NUM_COIN; i++){
		if(aCoinsPtr[i]!=NULL){
			aCoinsPtr[i]->nX==col;
			return 0;
		}
	}
	return 1;
}

unsigned char runCoins(unsigned char **t ){	
	unsigned char i,j;
	unsigned char val=0;	
	
	for(i=0; i < NUM_TILES_WIDTH; i++){
		for(j=0; j < NUM_TILES_HEIGHT; j++){
			if(t[i][j]>0 && (((j+1)%NUM_TILES_WIDTH)==0)){				
				val=val+t[i][j];
				t[i][j]=0;				
			}else if(t[i][j]>0){			
				t[i][j+1]=t[i][j];
			}		
		}
	}	
	SetCursor(14,20);
	printf("%u",val);
	GetChar();
	return val;
}

unsigned char isUpCoin(unsigned char row, unsigned char col, unsigned char ** table){
	unsigned char i;
	for(i = 0; i < MAX_NUM_COIN; i++){
		if(aCoinsPtr[i]!=NULL){
			if(aCoinsPtr[i]->nX==col){
				if(aCoinsPtr[i]->nY==row || aCoinsPtr[i]->nY+1==row){
					return tableCoins[col][aCoinsPtr[i]->nY];
				}
			}
		}
	}
	return 0;
}

unsigned char copyMatrix(unsigned char ** t, unsigned char ** table, _tSprite * v,  _tSprite * vCoins){	
	unsigned char val=0;
	for(i=0; i < NUM_TILES_WIDTH; i++){
		for(j=0; j < NUM_TILES_HEIGHT; j++){
			if(j+1!=NUM_TILES_HEIGHT){
				t[i][j+1]=table[i][j];
			}			
			if(v[i]->nY!=NUM_TILES_HEIGHT){
				vCoins[i]->nY=v[i]->nY;
				vCoins[i]->nX=v[i]->nX;
			}
			val=val+table[i][j];
		}
	}
	return val;
}

char total=0;
char betterPosition(char dist, unsigned char p1_nX, unsigned char p2_nX, unsigned char ** table, _tSprite * v){	
	char val=0, total;
	char i;
	unsigned char t[NUM_TILES_WIDTH][NUM_TILES_HEIGHT];
	_tSprite vCoins[MAX_NUM_COIN];
	val=copyMatrix(t, table, v, vCoins);
	if(val==0){
		return 0;
	}
	val=0;
	total=0;
	for(i = -dist; i <= dist; i++){
		if((per1.nX<per2.nX && per1.nX<per2.nX+i) || 
		   (per1.nX>per2.nX && per1.nX>per2.nX+i)){
			val=isUpCoin(per2.nY, per2.nX);
			if(!isEmptyCol(per2.nX+i) && per2.nX+i<NUM_TILES_WIDTH-1 && per2.nX+i>1 && (5==i || -5==i)){
				p2_nX=p2_nX+i;
				if((total=betterPosition(dist,p1_nX,p2_nX))<val){
					total=val;
				}
			}
		}
		
	}
	
	return val;
}
*/
////////////////////////////////////////////////////////////////////////
//Game
////////////////////////////////////////////////////////////////////////
void Game()
{
	unsigned int timeOut = 600;
    unsigned char *pScorePlayer1[2];
    unsigned char *pScorePlayer2[2];
  	unsigned char *pTime[2];
	unsigned int nLastMoveTime = GetTime();
	unsigned int nGameMovements = 0;
	unsigned int nGameSpeed = 10;
	unsigned char numCoinsMinus = 0;	
	unsigned char randomCoordX = 0;
	unsigned char freeCoin = 0;
	unsigned char nofreeCoin = 0;
  	unsigned int centsTime = 0;
  	unsigned int decsTime = 0;
  	unsigned int contDerP1 = 0;
  	unsigned int contIzqP1 = 0;
  	unsigned int contDerP2 = 0;
  	unsigned int contIzqP2 = 0;
  	unsigned int contArriP2 = 0;
  	unsigned int contArriP1 = 0;
  	unsigned int decsPoints1 = 0;
  	unsigned int decsPoints2 = 0;
  	unsigned int saltoP2 = 0;
  	unsigned int saltoP1 = 0;
  	unsigned int monedaCogida = 0;
	signed int beingHittedP1 = 0;
  	signed int beingHittedP2 = 0;
	unsigned char numCoins = 0;	
	unsigned char coinValue = 0;	
	int dif_X = 0, dif_Y = 0;
	unsigned int i;
	enum _eTileType eTileTypeAux;		
			
	pScorePlayer1[0] = (unsigned char *)0xC000 + ((SCORE_Y / 8) * 80) + ((SCORE_Y % 8) * 2048) + ((SCORE_X_1) / 8);
	pScorePlayer1[1] = (unsigned char *)0xC000 + ((SCORE_Y / 8) * 80) + ((SCORE_Y % 8) * 2048) + ((SCORE_X_1-NUMBER_WIDTH-1) / 8);
	pScorePlayer2[0] = (unsigned char *)0xC000 + ((SCORE_Y / 8) * 80) + ((SCORE_Y % 8) * 2048) + (SCORE_X_2 / 8);
	pScorePlayer2[1] = (unsigned char *)0xC000 + ((SCORE_Y / 8) * 80) + ((SCORE_Y % 8) * 2048) + ((SCORE_X_2-NUMBER_WIDTH-1) / 8);
  	pTime[0] = (unsigned char *)0xC000 + ((SCORE_Y / 8) * 80) + ((SCORE_Y % 8) * 2048) + (MID_SCREEN / 8);
  	pTime[1] = (unsigned char *)0xC000 + ((SCORE_Y / 8) * 80) + ((SCORE_Y % 8) * 2048) + ((MID_SCREEN-NUMBER_WIDTH-1) / 8);		
		
	numCoins=0;
	
	cpc_ScanKeyboard();	
	while(!cpc_TestKeyF(KEY_ESC)){
		//Player 2 Entra si marcamos la opcion de PLayer vs Player			
		cpc_ScanKeyboard();			
		if(modo==2){
			if(cpc_TestKeyF(KEY_ABA) && per1.nY==per2.nY){
				beingHittedP1 = actionPunchP2();
				if ((cpc_WyzTestPlayer()&8)==0) {
					cpc_WyzStartEffect(2,6);
				}
			}
			if(beingHittedP1!=0){
				if(beingHittedP1>0){
					throwCharLeftP1(beingHittedP1);
					beingHittedP1--;
				}else{
					throwCharRightP1(beingHittedP1);
					beingHittedP1++;
				}
			}
			if(cpc_TestKeyF(KEY_IZQ)){
				if(contIzqP2 == 0){
					if(per2.nX>1){
						if((per1.nX != per2.nX - 1 || per1.nY != per2.nY)){
							move_left(&per2, 1);
						}
					}
				}
				contIzqP2++;
				contDerP2 = 0;
				if(contIzqP2>100){
					contIzqP2 = 0;
				}
			}
			if(cpc_TestKeyF(KEY_DER)){
				if(contDerP2 == 0){
					if(per2.nX<NUM_TILES_WIDTH-2){
						if((per1.nX != per2.nX + 1 || per1.nY != per2.nY)){
							move_right(&per2, 1);
						}
					}
				}
				contDerP2++;
				contIzqP2 = 0;
				if(contDerP2>100){
					contDerP2=0;
				}
			}
			if(cpc_TestKeyF(KEY_ARR)){
				if(contArriP2 == 0){
					if(per1.nY != per2.nY - 1){
						jump(&per2);
						saltoP2 = 1;
					}
				}
			}
			if(saltoP2 == 1){
				contArriP2++;
				if(contArriP2>100){
					if(per2.nY+1!=per1.nY || per2.nX != per1.nX){
						normal(&per2);
						saltoP2 = 0;
						contArriP2=0;
					}
				}
			}				
		}
		//Player 1 
		
		if(cpc_TestKeyF(KEY_S) && per1.nY==per2.nY){
			beingHittedP2 = actionPunchP1();
			if ((cpc_WyzTestPlayer()&8)==0) {
				cpc_WyzStartEffect(2,6);
			}
		}
		if(beingHittedP2!=0){
			if(beingHittedP2>0){
				throwCharLeftP2(beingHittedP2);
				beingHittedP2--;
			}else{
				throwCharRightP2(beingHittedP2);
				beingHittedP2++;
			}
		}
		if(cpc_TestKeyF(KEY_A)){
			if(contIzqP1 == 0){
				if(per1.nX>1){
					if(per2.nX != per1.nX - 1 || per1.nY != per2.nY){
						move_left(&per1, 1);
					}
				}
			}
			contIzqP1++;
			contDerP1 = 0;
			if(contIzqP1>100){
				contIzqP1 = 0;
			}
		}
		if(cpc_TestKeyF(KEY_D)){
			if(contDerP1 == 0){
				if(per1.nX<NUM_TILES_WIDTH-2){
					if(per2.nX != per1.nX + 1 || per1.nY != per2.nY){
						move_right(&per1, 1);
					}
				}
			}
			contDerP1++;
			contIzqP1 = 0;
			if(contDerP1>100){
				contDerP1=0;
			}
		}
		if(cpc_TestKeyF(KEY_W)){
			if(contArriP1 == 0){
				if(per2.nY != per1.nY - 1){						
					jump(&per1);
					saltoP1 = 1;					
				}
			}
		}
		if(saltoP1 == 1){
			contArriP1++;
			if(contArriP1>100){
				if(per1.nY+1!=per2.nY || per2.nX != per1.nX){
					normal(&per1);
					saltoP1 = 0;
					contArriP1=0;
				}
			}
		}		
		
		numFrames++;
		if(numFrames==5){
			numFrames=0;
		}
		if(GetTime() - nLastMoveTime < nGameSpeed)
			continue;			
			
		freeCoin=0;
		if(numCoins<MAX_NUM_COIN){
			//Buscamos en el vector un hueco libre para generar otra moneda
			while(aCoinsPtr[freeCoin]!=NULL && freeCoin<MAX_NUM_COIN){
				freeCoin++;
			}
			if(freeCoin<MAX_NUM_COIN){
				//Creamos la moneda
				aCoinsPtr[freeCoin]=(tSpritePtr) malloc(sizeof(_tSprite));	
				//La colocamos con una coordenada X aleatoria que no coincida con el borde del escenario
				randomCoordX=(rand()%(NUM_TILES_WIDTH-3))+2;			
				//Comprobamos que las coordenadas de la nueva moneda no coincidan con otra que ya creada, evitamos solapamiento de sprites			
				while(aBackgroundTiles[randomCoordX][1]!=TileType_None){
					randomCoordX=(rand()%(NUM_TILES_WIDTH-3))+2;
				}			
				//Guardamos las coordenadas de la nueva moneda y la colocamos en el escenario para despues pintarla
				aCoinsPtr[freeCoin]->nX=randomCoordX;
				aCoinsPtr[freeCoin]->nY=1;			
				coinValue=(((rand()%100)>80) ? 5 : ((rand()%100)>60) ? 3 : 1);		
				tableCoins[aCoinsPtr[freeCoin]->nX][aCoinsPtr[freeCoin]->nY]=coinValue;				
				aBackgroundTiles[aCoinsPtr[freeCoin]->nX][aCoinsPtr[freeCoin]->nY] = ((coinValue==5) ? TileType_Coin_5 : (coinValue==3) ? TileType_Coin_3 : TileType_Coin);			
				DrawTile(aCoinsPtr[freeCoin]->nX, aCoinsPtr[freeCoin]->nY);		
				numCoins++;				
			}
		}	
		
		nofreeCoin=0;
		for(i = 0; i < numCoins; i++){
			//Buscamos una moneda que exista en el vector. Porque pueden haber huecos vacios en cualquier posicion
			while(aCoinsPtr[nofreeCoin]==NULL){
				nofreeCoin++;
			}
			//La moneda ahora tiene que bajar y primero tenemos que quitarle la posicion que tenia antes en la escena
			//y dibujarla lo que la eliminara de la escena temporalmente
			eTileTypeAux=aBackgroundTiles[aCoinsPtr[nofreeCoin]->nX][aCoinsPtr[nofreeCoin]->nY];
			aBackgroundTiles[aCoinsPtr[nofreeCoin]->nX][aCoinsPtr[nofreeCoin]->nY] = TileType_None;
			DrawTile(aCoinsPtr[nofreeCoin]->nX, aCoinsPtr[nofreeCoin]->nY);
			aCoinsPtr[nofreeCoin]->nY++;
			
			if(per1.nX==aCoinsPtr[nofreeCoin]->nX && aCoinsPtr[nofreeCoin]->nY==per1.nY){
				per1.nCoins=per1.nCoins+(eTileTypeAux == TileType_Coin? 1 : (eTileTypeAux == TileType_Coin_3) ? 3 : 5);
				monedaCogida=1;
			}else if(per2.nX==aCoinsPtr[nofreeCoin]->nX && aCoinsPtr[nofreeCoin]->nY==per2.nY){
				per2.nCoins=per2.nCoins+(eTileTypeAux == TileType_Coin? 1 : (eTileTypeAux == TileType_Coin_3) ? 3 : 5);
				monedaCogida=1;
			}
			
			//Canal de bot
			if(modo==1){
				
				//Modo Fácil
				if(dif==1){
					// Moneda a la izquierda
					if(per2.nX==aCoinsPtr[nofreeCoin]->nX+1 && (per2.nY-aCoinsPtr[nofreeCoin]->nY)<5){									
						if((per1.nX != per2.nX - 1 || per1.nY != per2.nY)){
							move_left(&per2, 1);
						}															
					}
					// Moneda a la derecha
					if(per2.nX==aCoinsPtr[nofreeCoin]->nX-1 && (per2.nY-aCoinsPtr[nofreeCoin]->nY)<5){													
						if((per1.nX != per2.nX + 1 || per1.nY != per2.nY)){
							move_right(&per2, 1);
						}															
					}
					if(per2.nY==aCoinsPtr[nofreeCoin]->nY-1){
						if(contArriP2 == 0){
							if(per1.nY != per2.nY - 1){
								jump(&per2);
								saltoP2 = 1;
							}
						}					
					}
					if(saltoP2 == 1){
						contArriP2++;
						if(contArriP2>100){
							if(per2.nY+1!=per1.nY || per2.nX != per1.nX){
								normal(&per2);
								saltoP2 = 0;
								contArriP2=0;
							}
						}
					}									
				}
				
				//Normal
				if(dif==2)
				{				
					dif_X=per2.nX-aCoinsPtr[nofreeCoin]->nX;
					dif_Y=per2.nY-aCoinsPtr[nofreeCoin]->nY;
					if(eTileTypeAux==TileType_Coin_5 && dif_Y<3 && per2.nX==aCoinsPtr[nofreeCoin]->nX){
						//Estate quieto
					}else if(per2.nX==aCoinsPtr[nofreeCoin]->nX+1 && dif_Y<3 && eTileTypeAux==TileType_Coin_5){													
						if(per1.nX != per2.nX - 1 || per1.nY != per2.nY){
							move_left(&per2, 1);
						}															
					}
					else if(per2.nX==aCoinsPtr[nofreeCoin]->nX-1 && dif_Y<3 && eTileTypeAux==TileType_Coin_5){												
						if((per1.nX != per2.nX + 1 || per1.nY != per2.nY)){
							move_right(&per2, 1);
						}
					}else if(eTileTypeAux==TileType_Coin_3 && dif_Y<2 && per2.nX==aCoinsPtr[nofreeCoin]->nX){
						//Estate quieto
					
					}else if(eTileTypeAux==TileType_Coin_5 && dif_Y<5 &&  abs(dif_X)<5){																	
						if(dif_X>0){
							if(per1.nX != per2.nX-abs(dif_X) || per1.nY != per2.nY){
								move_left(&per2, abs(dif_X));
							}
							
						}
						else if(dif_X<0){						
							if(per1.nX != per2.nX+abs(dif_X) || per1.nY != per2.nY){
								move_right(&per2, abs(dif_X));
							}							
						}					
					}
					else if(per2.nX==aCoinsPtr[nofreeCoin]->nX+1 && dif_Y<3 && eTileTypeAux==TileType_Coin_3){												
						if(per1.nX != per2.nX - 1 || per1.nY != per2.nY){
							move_left(&per2, 1);
						}													
					}
					else if(per2.nX==aCoinsPtr[nofreeCoin]->nX-1 && dif_Y<3 && eTileTypeAux==TileType_Coin_3){											
						if((per1.nX != per2.nX + 1 || per1.nY != per2.nY)){
							move_right(&per2, 1);
						}
					}
					else if(eTileTypeAux==TileType_Coin_3 && dif_Y<3 &&  abs(dif_X)<5 ){								
						if(dif_X>0){
							if(per1.nX != per2.nX-abs(dif_X) || per1.nY != per2.nY){
								move_left(&per2, abs(dif_X));
							}
							
						}else if(dif_X<0){						
							if(per1.nX != per2.nX + abs(dif_X) || per1.nY != per2.nY){
								move_right(&per2, abs(dif_X));
							}						
						}
					}
					
					else if(abs(dif_X)<5 && dif_Y<2){
						if(dif_X>0){
							if(per1.nX != per2.nX-abs(dif_X) || per1.nY != per2.nY){
								move_left(&per2, abs(dif_X));
							}							
						}else if(dif_X<0){							
							if(per1.nX != per2.nX + abs(dif_X) || per1.nY != per2.nY){
								move_right(&per2, abs(dif_X));
							}						
						}					
					}									
				}	

				// Dificil
				/*if(dif==3){					
					total=betterPosition(5,per1.nX,per2.nX,tableCoins);					
					if(total>0)move_right(&per2, total);
					if(total<0)move_left(&per2, total);
				}*/
			}
			
			//Si la moneda toca el suelo se elimina y si no se pinta con su nueva posicion bajando un tile
			if(aCoinsPtr[nofreeCoin]->nY==NUM_TILES_HEIGHT-2 || monedaCogida==1){
				numCoinsMinus++;										
				free(aCoinsPtr[nofreeCoin]);
				aCoinsPtr[nofreeCoin]=NULL;				
				tableCoins[aCoinsPtr[nofreeCoin]->nX][aCoinsPtr[nofreeCoin]->nY]=0;
				if(monedaCogida==1){
					if ((cpc_WyzTestPlayer()&8)==0) {
						cpc_WyzStartEffect(2,5);
					}
				}else{				
					if ((cpc_WyzTestPlayer()&8)==0) {
						cpc_WyzStartEffect(2,2);
					}
				}		
				monedaCogida = 0;
			}else{				
				aBackgroundTiles[aCoinsPtr[nofreeCoin]->nX][aCoinsPtr[nofreeCoin]->nY] = eTileTypeAux;		
				DrawTile(aCoinsPtr[nofreeCoin]->nX, aCoinsPtr[nofreeCoin]->nY);				
			}	
			//Aumentamos el numero de espacios ocupados en el vector
			nofreeCoin++;			
		}			
		numCoins=numCoins-numCoinsMinus;
		numCoinsMinus=0;
			
		nLastMoveTime = GetTime();
		nGameMovements++;

		if(nGameMovements % (nGameSpeed > 25 ? 20 : 40) == 0){
			if(nGameSpeed > 15)
				nGameSpeed -= 2;								
		}	

		//Points
		decsPoints1 = per1.nCoins / 10;
		if(decsPoints1>9)
		{
			timeOut = 0;
		}
	    PutSpriteMode0(pScorePlayer1[0], NUMBER_WIDTH_BYTES, NUMBER_HEIGHT, aNumberSprite[per1.nCoins-(decsPoints1*10)]);
	    if(decsPoints1>0){
			PutSpriteMode0(pScorePlayer1[1], NUMBER_WIDTH_BYTES, NUMBER_HEIGHT, aNumberSprite[decsPoints1]);
		}

		decsPoints2 = per2.nCoins / 10;
		if(decsPoints2>9)
		{
			timeOut = 0;
		}
	    PutSpriteMode0(pScorePlayer2[0], NUMBER_WIDTH_BYTES, NUMBER_HEIGHT, aNumberSprite[per2.nCoins-(decsPoints2*10)]);
	    if(decsPoints2>0){
			PutSpriteMode0(pScorePlayer2[1], NUMBER_WIDTH_BYTES, NUMBER_HEIGHT, aNumberSprite[decsPoints2]);
		}

		//Timing! Just a counter to finish the game 
		//Finish the game!!
		if(timeOut==0){
		  break;		
		}
		timeOut--;
		centsTime = timeOut / 100;
   	 	decsTime = (timeOut - centsTime*100) / 10;
    	PutSpriteMode0(pTime[0], NUMBER_WIDTH_BYTES, NUMBER_HEIGHT, aNumberSprite[decsTime]);
    	PutSpriteMode0(pTime[1], NUMBER_WIDTH_BYTES, NUMBER_HEIGHT, aNumberSprite[centsTime]);    	
	}

	for (i = 0; i < MAX_NUM_COIN; i++) {	
		if(aCoinsPtr[i]!=NULL)
			free(aCoinsPtr[i]);
	} 
	free(aCoinsPtr);
	
	if(!cpc_TestKeyF(KEY_ESC)){
		ShowMenuWinner(per1.nCoins, per2.nCoins);			
		while(!cpc_TestKeyF(KEY_ENT)){cpc_ScanKeyboard();}
	}
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//ShowMenu
////////////////////////////////////////////////////////////////////////
void ShowMenu()
{	
	SetMode(1);

	SetCursor(12, 5);
	printf("Thief vs Thief");

	SetCursor(9, 10);
	printf("Press 1: Player vs Bot");

	SetCursor(8, 14);
	printf("Press 2: Player vs Player");

	SetCursor(11, 24);
	printf("The Beginners - 2013");
	
	modo=0;
	while (!modo) {
		cpc_ScanKeyboard();
		if(cpc_TestKeyF(KEY_1)){			
			modo=1;				
		}else if(cpc_TestKeyF(KEY_2)){					
			modo=2;		
		}
	}		
}
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//ShowMenu
////////////////////////////////////////////////////////////////////////
void ShowMenuSub()
{
	SetMode(1);

	if(modo==2)
	{
	
		SetCursor(12, 5);
		printf("Multiplayer");

		SetCursor(3, 10);
		printf("Player 1: A: Left - D: Right - W: Jump - S: Punch");

		SetCursor(2, 14);
		printf("Player 2: Arrows (left, right, jump) & S: Punch");

		SetCursor(8, 20);
		printf("Press 'ENTER' to enjoy");
				
		while(!cpc_TestKeyF(KEY_ENT)){ cpc_ScanKeyboard(); }
	}
	if(modo==1)
	{
		SetCursor(13, 5);
		printf("Player vs Bot");

		SetCursor(11, 8);
		printf("How can you move?");
		SetCursor(1, 10);
		printf("A: Left - D: Right - W: Jump - S: Punch");
		
		SetCursor(7, 14);
		printf("Choose difficulty:");
		SetCursor(14,16);
		printf("Easy(1)");
		SetCursor(14,18);
		printf("Medium(2)");
		SetCursor(14,20);
		printf("Hard(3)");
		
		dif=0;
		while(!dif){ 
			cpc_ScanKeyboard(); 
			if(cpc_TestKeyF(KEY_1)){
				dif=1;
			}else if(cpc_TestKeyF(KEY_2)){
				dif=2;	
			}else if(cpc_TestKeyF(KEY_3)){			
				dif=3;				
			}	
		}		
	}
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//main
////////////////////////////////////////////////////////////////////////
void main()
{		
unsigned char i =0, j=0;
	cpc_DisableFirmware();		
	
	cpc_WyzInitPlayer(SOUND_TABLE, RULE_TABLE, EFFECT_TABLE, SONG_TABLE);		//Iniciamos los datos
		
	// Batería por el canal C:	
	cpc_WyzConfigurePlayer (0);
	
	// Ponemos el buffer en un sitio adecuado:	
	//cpc_WyzSetBuffer(0x9999);
	
	cpc_AssignKey (KEY_IZQ, 0x4101);		// CUR LEFT
	cpc_AssignKey (KEY_DER, 0x4002);		// CUR RIGHT
	cpc_AssignKey (KEY_ARR, 0x4001);		// CUR UP
	cpc_AssignKey (KEY_ABA, 0x4004);		// CUR DOWN
	cpc_AssignKey (KEY_FIR, 0x4580);		// SPACE
	cpc_AssignKey (KEY_A, 0x4820);			// A
	cpc_AssignKey (KEY_W, 0x4708);			// W
	cpc_AssignKey (KEY_D, 0x4720);			// D
	cpc_AssignKey (KEY_S, 0x4710);			// S
	cpc_AssignKey (KEY_1, 0x4801);			// 1
	cpc_AssignKey (KEY_2, 0x4802);			// 2
	cpc_AssignKey (KEY_3, 0x4702);			// 3
	cpc_AssignKey (KEY_4, 0x4701);			// 4	
	cpc_AssignKey (KEY_RET, 0x4604);		// R
	cpc_AssignKey (KEY_ENT, 0x4040);		// ENTER
	cpc_AssignKey (KEY_ESC, 0x4804);		// ESC
	
	
	//Seleccionamos la cancion
	cpc_WyzLoadSong(0);			
	cpc_WyzSetPlayerOn();		//Iniciamos la musica y los efectos de sonido	
	cpc_ScanKeyboard();		
	while(!cpc_TestKeyF(KEY_ESC)){
		SetMode(1);		
		ShowMenu();					
		ShowMenuSub();
		InitGame();	
		Game();
	}
	
	cpc_WyzSetPlayerOff();	//stop music and SFX
	cpc_EnableFirmware();
}
////////////////////////////////////////////////////////////////////////